var total,
    steps,
    errors;


exports.check = function( obj, model, options ) {

    total  = 0;
    steps  = 0;
    errors = [];

    walk( obj, model, { action: function() { total++; steps++; } } );
    walk( obj, model, options );

}


function walk( obj, model, options ) {
    if ( options ) {
        options.action = options.action || checkTypes;
    } else {
        var options = {
            action: checkTypes
        }
    }

    if ( Array.isArray(model) ) {
        handleArray( obj, model, options );
    }
    else if ( typeof model === 'object' ) {
        handleObject( obj, model, options );
    }
    // string, number, boolean or function
    else {
        options.action( obj, model, options );
    } 
}


function handleArray( obj, model, options ) {
    var lng = model.length;

    if ( typeof model === 'string' ) {
        options.action( obj, model, options );
    } else {
        for ( var i = 0; i < lng; i++ ) {
            walk( obj[i], model[i], options );
        }
    }
}


function handleObject( obj, model, options ) {
    for ( var key in model ) {
        if ( typeof obj[key] === 'undefined' ) {
            options.action( obj[key], model[key], options );
        }

        else if ( typeof model[key] === 'string' ) {
            options.action( obj[key], model[key], options );
        }

        else {
            walk( obj[key], model[key], options );
        }
    }
}


function checkTypes( obj, model, options ) {
    var isSameType,
        error = 'expected ' + obj + ' to be ' + model;

    if ( Array.isArray(obj) ) {
        isSameType = 'array' === model;
    } else {
        isSameType = typeof obj === model;
    }

    if ( !isSameType ) errors.push( error );

    if ( typeof options.assert === 'function' ) options.assert( isSameType, error );

    finish( options );
}


function finish( options ) {
    steps--;

    if ( steps === 0 && typeof options.callback === 'function' ) {
        var error = ( errors.length > 0 ) ? errors : null;

        options.callback(error, {
            total : total,
            passed: total - errors.length,
            errors: errors
        });
    }
}
